CREATE OR REPLACE TYPE PPI.PROGRAMACION_TABLE_TYPE AS TABLE OF PROGRAMACION_TYPE;

GRANT EXECUTE ON PPI.PROGRAMACION_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL(

    --periodoAnual: parametro de ingreso
    periodoAnual IN NUMBER
    
)
--RETURN: tabla virtual de tipo APD_TABLE_TYPE
RETURN PROGRAMACION_TABLE_TYPE PIPELINED AS

    --***************************************************************
    --VARIABLES
    --***************************************************************
    -- Almacena los años en un arreglo
    v_periodos  PERIODO_ANUAL_TABLE_TYPE := PERIODO_ANUAL_TABLE_TYPE();

    -- Variable para almacenar el año actual
    v_periodo PERIODO_ANUAL_TYPE;


    -- Almacena los años en un arreglo
    v_anos ANO_TABLE_TYPE := ANO_TABLE_TYPE();

    -- Variable para almacenar el año actual
    v_ano ANO_TYPE;

    -- Variable para construir la consulta PIVOT
    v_sql CLOB;

    -- Variable para almacenar el resultado de la consulta PIVOT
    v_result SYS_REFCURSOR;

    -- Variables para almacenar los valores dinámicos de las columnas del PIVOT
    v_aporte            NUMBER;
    v_cantidad_aporte   NUMBER;
    v_representatividad NUMBER;
    v_programacion_1    NUMBER;
    v_programacion_2    NUMBER;
    v_programacion_3    NUMBER;
    v_programacion_4    NUMBER;
    v_programacion_5    NUMBER;

BEGIN
    --***************************************************************
    --OBTENCIÓN DE DATOS
    --***************************************************************
    -- Ejecutar consulta del ID_PERIODO_ANUAL y  ANO para almacenarlos en arreglos para crear el PIVOT 
    FOR rec IN (SELECT ANO, ID_PERIODO_ANUAL
                FROM PERIODO_ANUAL
                WHERE ID_PERIODO_ANUAL IN (SELECT PERIODO_ANUAL
                                           FROM TABLE(PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(periodoAnual)))
                ORDER BY ANO)
    LOOP
        -- Utilizar el constructor del tipo ANO_TYPE
        v_ano := ANO_TYPE(rec.ANO);

        -- Utilizar el constructor del tipo ANO_TYPE
        v_periodo := PERIODO_ANUAL_TYPE(rec.ID_PERIODO_ANUAL);

       -- Añadir los resultados al arreglo de años
        v_periodos.EXTEND;
        v_periodos(v_periodos.LAST) := v_periodo;

        -- Añadir los resultados al arreglo de años
        v_anos.EXTEND;
        v_anos(v_anos.LAST) := v_ano;

    END LOOP;


   v_sql := 'SELECT * FROM ( ';
    v_sql := v_sql || 'SELECT ';
    v_sql := v_sql || 'DISTINCT ';
    v_sql := v_sql || 'APD.APORTE_ESTRATEGICO                                          APORTE_ESTRATEGICO, ';
    v_sql := v_sql || 'APD.PERIODO_ANUAL                                               PERIODO_ANUAL, ';
    v_sql := v_sql || 'APD.DISTRIBUCION_PORCENTUAL*(1 / CAN_APO.CANTIDAD_APORTES)      PROGRAMACION, ';
    v_sql := v_sql || 'CAN_APO.CANTIDAD_APORTES                                        CANTIDAD_APORTES_X_ACCION, ';
    v_sql := v_sql || '(1 / CAN_APO.CANTIDAD_APORTES)                                  REPRESENTATIVIDAD';
    v_sql := v_sql || ' FROM ';
    v_sql := v_sql || 'META_POA_APORTE                         MPAP ';
    v_sql := v_sql || 'INNER JOIN (SELECT * FROM TABLE(PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(' || periodoAnual || '))) METAS_FUNCIONALES on MPAP.META_POA = METAS_FUNCIONALES.META_POA ';
    v_sql := v_sql || 'INNER JOIN APORTE_ESTRATEGICO APE ON (APE.ID_APORTE_ESTRATEGICO = MPAP.APORTE_ESTRATEGICO) ';
    v_sql := v_sql || 'INNER JOIN APORTE_PAE_DISTRIBUCION APD ON (APD.APORTE_ESTRATEGICO = APE.ID_APORTE_ESTRATEGICO) ';
    v_sql := v_sql || 'INNER JOIN ACCION_ESTRATEGICA ACE ON (ACE.ID_ACCION_ESTRATEGICA = APE.ACCION_ESTRATEGICA) ';
    v_sql := v_sql || 'INNER JOIN (SELECT ACCION_ESTRATEGICA, COUNT(*) AS CANTIDAD_APORTES FROM PPI.APORTE_ESTRATEGICO GROUP BY ACCION_ESTRATEGICA) CAN_APO ON CAN_APO.ACCION_ESTRATEGICA = ACE.ID_ACCION_ESTRATEGICA ';
    v_sql := v_sql || 'WHERE ';
    v_sql := v_sql || 'APD.PERIODO_ANUAL IN (SELECT * FROM TABLE(PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(' || periodoAnual || '))) ';
    v_sql := v_sql || ') ';
    v_sql := v_sql || 'PIVOT ( ';
    v_sql := v_sql || 'MAX(PROGRAMACION) FOR PERIODO_ANUAL IN ( ';


    -- CONSTRUIR PIVOTEO
    FOR i IN 1..v_anos.LAST LOOP

        v_sql := v_sql || v_periodos(i).PERIODO_ANUAL || ' as "' || v_anos(i).ANO || '"';

        IF i < v_anos.LAST THEN
            v_sql := v_sql || ', ';
        END IF;
    END LOOP;

    v_sql := v_sql || ') ';
    v_sql := v_sql || ') ';
    v_sql := v_sql || 'ORDER BY APORTE_ESTRATEGICO';
    DBMS_OUTPUT.PUT_LINE('DEBUG: v_sql: ' || v_sql);


    -- Ejecutar la consulta PIVOT
    OPEN v_result FOR v_sql;

    --**************************************************************
    --RECUPERAR Y FORMATEAAR LOS DATOS
    --**************************************************************
    LOOP
       FETCH v_result INTO v_aporte,v_cantidad_aporte, v_representatividad, v_programacion_1, v_programacion_2, v_programacion_3, v_programacion_4, v_programacion_5;
       EXIT WHEN v_result%NOTFOUND;

       PIPE ROW(PROGRAMACION_TYPE(v_aporte,v_cantidad_aporte, v_representatividad, v_programacion_1, v_programacion_2, v_programacion_3, v_programacion_4, v_programacion_5));
    END LOOP;

    CLOSE v_result;

    RETURN;

END OBTENER_PROGRAMACION_X_PERIODO_ANUAL;