--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_METAS_OPE_FUCIONALES_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.META_POA_TABLE_TYPE;
DROP TYPE PPI.META_POA_TYPE;
DROP FUNCTION PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato META_POA
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.META_POA_TYPE AS OBJECT (
    META_POA NUMBER
);

GRANT EXECUTE ON PPI.META_POA_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato MetaPoaType
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.META_POA_TABLE_TYPE AS TABLE OF META_POA_TYPE;

GRANT EXECUTE ON PPI.META_POA_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN META_POA_TABLE_TYPE PIPELINED AS

BEGIN
    FOR rec IN (
        SELECT 
        DISTINCT
        MPE.META_POA
        FROM META_POA_EVALUACION MPE
        INNER JOIN META_POA MP ON (MP.ID_META_POA = MPE.META_POA)
        INNER JOIN OBJETIVO_POA OP ON (OP.ID_OBJETIVO_POA = MP.OBJETIVO_POA)
        INNER JOIN PLAN_OPERATIVO PO ON (PO.ID_PLAN_OPERATIVO = OP.PLAN_OPERATIVO)
        INNER JOIN UNIDAD_EJECUTORA UE_PO ON (UE_PO.ID_UNIDAD_EJECUTORA = PO.UNIDAD_EJECUTORA)
        INNER JOIN EVALUACION_OPERATIVA EO ON (EO.PLAN_OPERATIVO = PO.ID_PLAN_OPERATIVO)
        WHERE PO.PERIODO_ANUAL = periodoAnual
        AND PO.ESTADO_FORMULACION_OPE IN (SELECT VALOR FROM PARAMETRO WHERE LLAVE IN ('param_ppi_ppo_estadoFormulacionOpe_aprobado_unSoloNivel', 'param_ppi_ppo_estadoFormulacionOpe_aprobado_unidad_integradora'))
        AND PO.TIPO_PLAN_OPERATIVO = (SELECT VALOR FROM PARAMETRO WHERE LLAVE = 'param_ppi_ppo_tipoPlanOperativo_unidad')
        AND EO.ESTADO_EVALUACION_POA IN (SELECT VALOR FROM PARAMETRO WHERE LLAVE IN ('param_ppi_ppo_estadoEvaluacionPOA_aprobado_unSoloNivel', 'param_ppi_ppo_estadoEvaluacionPOA_aprobado_unidad_integradora'))
        AND MPE.ETAPA_PLAN = EO.ETAPA_PLAN
        --AND EO.FECHA_APROBACION = (
        --    SELECT 
         --   MAX(EO.FECHA_APROBACION) 
         --   FROM 
         --   EVALUACION_OPERATIVA EO1
         --   INNER JOIN PLAN_OPERATIVO PO1 ON (PO1.ID_PLAN_OPERATIVO = EO1.PLAN_OPERATIVO)
         --   WHERE 
         --   PO1.ID_PLAN_OPERATIVO = PO.ID_PLAN_OPERATIVO
         --   AND PO1.UNIDAD_EJECUTORA = PO.UNIDAD_EJECUTORA
        --)
        AND EO.FECHA_APROBACION = (
            SELECT 
                MAX(EO.FECHA_APROBACION) 
            FROM 
            EVALUACION_OPERATIVA EO1
            INNER JOIN PLAN_OPERATIVO PO1 ON (PO1.ID_PLAN_OPERATIVO = EO1.PLAN_OPERATIVO)
            WHERE 
            PO1.ID_PLAN_OPERATIVO = PO.ID_PLAN_OPERATIVO
            AND PO1.UNIDAD_EJECUTORA = PO.UNIDAD_EJECUTORA
        )
        ORDER BY MPE.META_POA
    ) LOOP
        PIPE ROW (META_POA_TYPE(rec.META_POA));
    END LOOP;

    RETURN;

END OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS MetaPoa
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(202));








--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_PERIODOS_ANUALES_DISTRIBUCION_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.PERIODO_ANUAL_TABLE_TYPE;
DROP TYPE PPI.PERIODO_ANUAL_TYPE;
DROP FUNCTION PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL;
DROP SYNONYM OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato PERIODO_ANUAL_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.PERIODO_ANUAL_TYPE AS OBJECT (
    PERIODO_ANUAL NUMBER
);

GRANT EXECUTE ON PPI.PERIODO_ANUAL_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PERIODO_ANUAL_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.PERIODO_ANUAL_TABLE_TYPE AS TABLE OF PERIODO_ANUAL_TYPE;

GRANT EXECUTE ON PPI.PERIODO_ANUAL_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN PERIODO_ANUAL_TABLE_TYPE PIPELINED AS

BEGIN
    FOR rec IN (
        SELECT 
        DISTINCT
        APD.PERIODO_ANUAL
        FROM
        META_POA_APORTE MPAP
        INNER JOIN (SELECT *
        FROM TABLE(PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(periodoAnual))) METAS_FUNCIONALES on MPAP.META_POA = METAS_FUNCIONALES.META_POA
        INNER JOIN APORTE_ESTRATEGICO APE ON (APE.ID_APORTE_ESTRATEGICO = MPAP.APORTE_ESTRATEGICO)
        INNER JOIN APORTE_PAE_DISTRIBUCION APD ON (APD.APORTE_ESTRATEGICO = APE.ID_APORTE_ESTRATEGICO)
        ORDER BY APD.PERIODO_ANUAL
    ) LOOP
        PIPE ROW (PERIODO_ANUAL_TYPE(rec.PERIODO_ANUAL));
    END LOOP;

    RETURN;

END OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

GRANT EXECUTE ON  PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(202));






--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_APD_X_PERIODDO_ANUAL
--*******************************************************************************
--*******************************************************************************


--*****************************************************
--COMPONENTE REQUERIDO: AÑO
--*****************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.ANO_TABLE_TYPE;
DROP TYPE PPI.ANO_TYPE;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ANO_TYPE AS OBJECT (
    ANO NUMBER(4)
);
GRANT EXECUTE ON PPI.ANO_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato ANO_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ANO_TABLE_TYPE AS TABLE OF ANO_TYPE;

GRANT EXECUTE ON PPI.ANO_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;





--*****************************************************
--FUNCIÓN: OBTENER_APD_X_PERIODDO_ANUAL
--*****************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.APD_TABLE_TYPE;
DROP TYPE PPI.APD_TYPE;
DROP FUNCTION PPI.OBTENER_APD_X_PERIODO_ANUAL;
DROP SYNONYM OBTENER_APD_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE  PPI.APD_TYPE AS OBJECT (
    APORTE_ESTRATEGICO  NUMBER,
    ANNO_1              NUMBER,
    ANNO_2              NUMBER,
    ANNO_3              NUMBER,
    ANNO_4              NUMBER,
    ANNO_5              NUMBER
);
GRANT EXECUTE ON PPI.APD_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.APD_TABLE_TYPE AS TABLE OF APD_TYPE;

GRANT EXECUTE ON PPI.APD_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION PPI.OBTENER_APD_X_PERIODO_ANUAL(
    --periodoAnual: parametro de ingreso
    periodoAnual IN NUMBER
)
--RETURN: tabla virtual de tipo APD_TABLE_TYPE
RETURN APD_TABLE_TYPE PIPELINED AS

    --***************************************************************
    --VARIABLES
    --***************************************************************
    -- Almacena los años en un arreglo
    v_periodos  PERIODO_ANUAL_TABLE_TYPE := PERIODO_ANUAL_TABLE_TYPE();

    -- Variable para almacenar el año actual
    v_periodo PERIODO_ANUAL_TYPE;


    -- Almacena los años en un arreglo
    v_anos ANO_TABLE_TYPE := ANO_TABLE_TYPE();

    -- Variable para almacenar el año actual
    v_ano ANO_TYPE;

    -- Variable para construir la consulta PIVOT
    v_sql CLOB;

    -- Variable para almacenar el resultado de la consulta PIVOT
    v_result SYS_REFCURSOR;

    -- Variables para almacenar los valores dinámicos de las columnas del PIVOT
    v_aporte NUMBER;
    v_anno_1 NUMBER;
    v_anno_2 NUMBER;
    v_anno_3 NUMBER;
    v_anno_4 NUMBER;
    v_anno_5 NUMBER;

BEGIN
    --***************************************************************
    --OBTENCIÓN DE DATOS
    --***************************************************************
    -- Ejecutar consulta del ID_PERIODO_ANUAL y  ANO para almacenarlos en arreglos para crear el PIVOT 
    FOR rec IN (SELECT ANO, ID_PERIODO_ANUAL
                FROM PERIODO_ANUAL
                WHERE ID_PERIODO_ANUAL IN (SELECT PERIODO_ANUAL
                                           FROM TABLE(PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(periodoAnual)))
                ORDER BY ANO)
    LOOP
        -- Utilizar el constructor del tipo ANO_TYPE
        v_ano := ANO_TYPE(rec.ANO);

        -- Utilizar el constructor del tipo ANO_TYPE
        v_periodo := PERIODO_ANUAL_TYPE(rec.ID_PERIODO_ANUAL);

       -- Añadir los resultados al arreglo de años
        v_periodos.EXTEND;
        v_periodos(v_periodos.LAST) := v_periodo;

        -- Añadir los resultados al arreglo de años
        v_anos.EXTEND;
        v_anos(v_anos.LAST) := v_ano;

    END LOOP;


    --**************************************************************
    --CONSULTA DINÁMICA
    --**************************************************************
    v_sql := 'SELECT * FROM (SELECT APD.APORTE_ESTRATEGICO, APD.PERIODO_ANUAL, APD.DISTRIBUCION_PORCENTUAL ';
    v_sql := v_sql || 'FROM META_POA_APORTE MPAP ';
    v_sql := v_sql || 'INNER JOIN (SELECT * FROM TABLE(PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(' || periodoAnual || '))) METAS_FUNCIONAES ';
    v_sql := v_sql || 'ON MPAP.META_POA = METAS_FUNCIONAES.META_POA ';
    v_sql := v_sql || 'INNER JOIN APORTE_ESTRATEGICO APE ON APE.ID_APORTE_ESTRATEGICO = MPAP.APORTE_ESTRATEGICO ';
    v_sql := v_sql || 'INNER JOIN APORTE_PAE_DISTRIBUCION APD ON APD.APORTE_ESTRATEGICO = APE.ID_APORTE_ESTRATEGICO ';
    v_sql := v_sql || 'WHERE APD.PERIODO_ANUAL IN (SELECT * FROM TABLE(PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(' || periodoAnual || ')))) ';
    v_sql := v_sql || 'PIVOT (MAX(DISTRIBUCION_PORCENTUAL) FOR PERIODO_ANUAL IN (';

    FOR i IN 1..v_anos.LAST LOOP
        v_sql := v_sql || v_periodos(i).PERIODO_ANUAL || ' as "' || v_anos(i).ANO || '"';
        IF i < v_anos.LAST THEN
            v_sql := v_sql || ', ';
        END IF;
    END LOOP;

    v_sql := v_sql || ')) ORDER BY APORTE_ESTRATEGICO';


    -- Ejecutar la consulta PIVOT
    OPEN v_result FOR v_sql;

    --**************************************************************
    --RECUPERAR Y FORMATEAAR LOS DATOS
    --**************************************************************
    LOOP
       FETCH v_result INTO v_aporte,v_anno_1, v_anno_2, v_anno_3, v_anno_4, v_anno_5;
       EXIT WHEN v_result%NOTFOUND;

       PIPE ROW(APD_TYPE(v_aporte,v_anno_1, v_anno_2, v_anno_3, v_anno_4, v_anno_5));
    END LOOP;

    CLOSE v_result;

    RETURN;

END OBTENER_APD_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++


--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_APD_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_APD_X_PERIODO_ANUAL(202));
























--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_PROGRAMACION_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.PROGRAMACION_TABLE_TYPE;
DROP TYPE PPI.PROGRAMACION_TYPE;
DROP FUNCTION PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL;
DROP SYNONYM OBTENER_PROGRAMACION_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.PROGRAMACION_TYPE AS OBJECT (
    APORTE_ESTRATEGICO          NUMBER,
    DISTRIBUCION_PORCENTUAL     NUMBER,
    CANTIDAD_APORTES            NUMBER,
    REPRESENTATIVIDAD           NUMBER,
    PROGRAMACION                NUMBER
);

GRANT EXECUTE ON PPI.PROGRAMACION_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.PROGRAMACION_TABLE_TYPE AS TABLE OF PROGRAMACION_TYPE;

GRANT EXECUTE ON PPI.PROGRAMACION_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++


CREATE OR REPLACE FUNCTION  PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN PROGRAMACION_TABLE_TYPE PIPELINED AS

BEGIN
    FOR rec IN (
        
        SELECT 
        DISTINCT 
        APD.APORTE_ESTRATEGICO                                          APORTE_ESTRATEGICO,
        APD.DISTRIBUCION_PORCENTUAL                                     DISTRIBUCION_PORCENTUAL,
        CAN_APO.CANTIDAD_APORTES                                        CANTIDAD_APORTES_X_ACCION,
        (1 / CAN_APO.CANTIDAD_APORTES)                                  REPRESENTATIVIDAD,
        APD.DISTRIBUCION_PORCENTUAL*(1 / CAN_APO.CANTIDAD_APORTES)      PROGRAMACION
        FROM
        META_POA_APORTE                         MPAP
        INNER JOIN (SELECT * FROM TABLE(PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(periodoAnual))) METAS_FUNCIONALES on MPAP.META_POA = METAS_FUNCIONALES.META_POA
        INNER JOIN APORTE_ESTRATEGICO           APE ON (APE.ID_APORTE_ESTRATEGICO = MPAP.APORTE_ESTRATEGICO)
        INNER JOIN APORTE_PAE_DISTRIBUCION      APD ON (APD.APORTE_ESTRATEGICO = APE.ID_APORTE_ESTRATEGICO)
        INNER JOIN ACCION_ESTRATEGICA           ACE ON (ACE.ID_ACCION_ESTRATEGICA = APE.ACCION_ESTRATEGICA)
        INNER JOIN (SELECT ACCION_ESTRATEGICA, COUNT(*) AS CANTIDAD_APORTES FROM PPI.APORTE_ESTRATEGICO GROUP BY ACCION_ESTRATEGICA) CAN_APO ON CAN_APO.ACCION_ESTRATEGICA = ACE.ID_ACCION_ESTRATEGICA
        WHERE APD.PERIODO_ANUAL = periodoAnual
        ORDER BY 
        APD.APORTE_ESTRATEGICO 
    ) LOOP
        PIPE ROW(PROGRAMACION_TYPE(REC.APORTE_ESTRATEGICO,REC.DISTRIBUCION_PORCENTUAL,REC.CANTIDAD_APORTES_X_ACCION, REC.REPRESENTATIVIDAD, REC.PROGRAMACION));
    END LOOP;

    RETURN;

END OBTENER_PROGRAMACION_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL(202));








--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_ULTIMA_ETAPA_PLAN_EVALUACION_FINALIZADA
--*******************************************************************************
--*******************************************************************************

CREATE OR REPLACE FUNCTION  PPI.OBTENER_ULTIMA_ETAPA_PLAN_EVALUACION_FINALIZADA(
    periodoAnual IN NUMBER
)
RETURN NUMBER
IS
    v_id_etapa_plan NUMBER;
    
BEGIN

    SELECT
        EP.ID_ETAPA_PLAN INTO v_id_etapa_plan
    FROM ETAPA_PLAN EP
    WHERE
    EP.TIPO_ETAPA = (SELECT VALOR FROM PARAMETRO WHERE LLAVE = 'param_ppi_ppo_tipoEtapa_evaluacion')
    AND EP.PERIODO_ANUAL = periodoAnual
    AND EP.TIPO_PLAN_OPERATIVO = (SELECT VALOR FROM PARAMETRO WHERE LLAVE = 'param_ppi_ppo_tipoPlanOperativo_unidad')
    AND EP.FECHA_FINAL_EVALUACION <= TRUNC(SYSDATE)
    ORDER BY
    EP.FECHA_FINAL_EVALUACION DESC
    FETCH FIRST 1 ROW ONLY;

  RETURN v_id_etapa_plan;
END OBTENER_ULTIMA_ETAPA_PLAN_EVALUACION_FINALIZADA;

--PROBAR CONSULTA
SELECT PPI.OBTENER_ULTIMA_ETAPA_PLAN_EVALUACION_FINALIZADA(:periodoAnual) AS ULTIMA_ETAPA_PLAN_EVALUACION FROM DUAL;


























--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.PROMEDIO_AVANCE_APORTE_TABLE_TYPE;
DROP TYPE PPI.PROMEDIO_AVANCE_APORTE_TYPE;
DROP FUNCTION PPI.OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato PERIODO_ANUAL_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.PROMEDIO_AVANCE_APORTE_TYPE AS OBJECT (
    APORTE_ESTRATEGICO  NUMBER,
    PROMEDIO            NUMBER,
    ETAPA_PLAN          NUMBER
);

GRANT EXECUTE ON PPI.PROMEDIO_AVANCE_APORTE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PERIODO_ANUAL_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.PROMEDIO_AVANCE_APORTE_TABLE_TYPE AS TABLE OF PROMEDIO_AVANCE_APORTE_TYPE;

GRANT EXECUTE ON PPI.PROMEDIO_AVANCE_APORTE_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN PROMEDIO_AVANCE_APORTE_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT 
                MPA.APORTE_ESTRATEGICO,
                NVL(AVG(MPE.EJECUCION),0) PROMEDIO,
                MPE.ETAPA_PLAN
                FROM
                META_POA_APORTE MPA
                INNER JOIN (
                    SELECT * FROM TABLE(
                        PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(periodoAnual)
                    )
                ) METAS_FUNCIONALES ON MPA.META_POA = METAS_FUNCIONALES.META_POA
                INNER JOIN META_POA_EVALUACION MPE ON (MPE.META_POA = MPA.META_POA)
                INNER JOIN ETAPA_PLAN EP ON (EP.ID_ETAPA_PLAN = MPE.ETAPA_PLAN)
                WHERE 
                EP.PERIODO_ANUAL = periodoAnual
                AND MPE.ETAPA_PLAN = PPI.OBTENER_ULTIMA_ETAPA_PLAN_EVALUACION_FINALIZADA(periodoAnual)
                GROUP BY MPA.APORTE_ESTRATEGICO, MPE.ETAPA_PLAN
                ORDER BY 
                MPA.APORTE_ESTRATEGICO
    ) 
    LOOP
    PIPE ROW (PROMEDIO_AVANCE_APORTE_TYPE(rec.APORTE_ESTRATEGICO,rec.PROMEDIO, rec.ETAPA_PLAN));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL;

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

GRANT EXECUTE ON  PPI.OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL(:periodoAnual));
















--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_EJECUCION_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.EJECUCION_TABLE_TYPE;
DROP TYPE PPI.EJECUCION_TYPE;
DROP FUNCTION PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.EJECUCION_TYPE AS OBJECT (
    APORTE_ESTRATEGICO          NUMBER,
    PROGRAMACION                NUMBER,
    PROMEDIO                    NUMBER,
    ETAPA_PLAN                  NUMBER,
    EJECUCION                   NUMBER
);

GRANT EXECUTE ON PPI.EJECUCION_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.EJECUCION_TABLE_TYPE AS TABLE OF EJECUCION_TYPE;

GRANT EXECUTE ON PPI.EJECUCION_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN EJECUCION_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT 
                PROGRAMACION.APORTE_ESTRATEGICO                                 APORTE_ESTRATEGICO,
                PROGRAMACION.PROGRAMACION                                       PROGRAMACION,
                PROMEDIO.PROMEDIO                                               PROMEDIO,
                PROMEDIO.ETAPA_PLAN                                             ETAPA_PLAN,
                ((PROGRAMACION.PROGRAMACION*PROMEDIO.PROMEDIO)/100)             EJECUCION
            FROM TABLE(PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL(periodoAnual)) PROGRAMACION
            
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL(periodoAnual))
            ) PROMEDIO ON PROGRAMACION.APORTE_ESTRATEGICO = PROMEDIO.APORTE_ESTRATEGICO
            WHERE 
            PROMEDIO.ETAPA_PLAN = PPI.OBTENER_ULTIMA_ETAPA_PLAN_EVALUACION_FINALIZADA(periodoAnual)
            ORDER BY 
            PROGRAMACION.APORTE_ESTRATEGICO
    ) 
    LOOP
    PIPE ROW (EJECUCION_TYPE(rec.APORTE_ESTRATEGICO,rec.PROGRAMACION, rec.PROMEDIO,rec.ETAPA_PLAN,rec.EJECUCION));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_EJECUCION_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL(:periodoAnual));












--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_ACUMULADO_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.ACUMULADO_TABLE_TYPE;
DROP TYPE PPI.ACUMULADO_TYPE;
DROP FUNCTION PPI.OBTENER_ACUMULADO_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_TYPE AS OBJECT (
    APORTE_ESTRATEGICO          NUMBER,
    ACUMULADO                   NUMBER
);

GRANT EXECUTE ON PPI.ACUMULADO_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato ACUMULADO_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_TABLE_TYPE AS TABLE OF ACUMULADO_TYPE;

GRANT EXECUTE ON PPI.ACUMULADO_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION PPI.OBTENER_ACUMULADO_X_PERIODO_ANUAL(
    --periodoAnual: parametro de ingreso
    periodoAnual IN NUMBER
)
--RETURN: tabla virtual de tipo APD_TABLE_TYPE
RETURN ACUMULADO_TABLE_TYPE PIPELINED AS

    --***************************************************************
    --VARIABLES
    --***************************************************************
    -- Almacena los años en un arreglo
    v_periodos  PERIODO_ANUAL_TABLE_TYPE := PERIODO_ANUAL_TABLE_TYPE();

    -- Variable para almacenar el año actual
    v_periodo PERIODO_ANUAL_TYPE;



    -- Variable para construir la consulta PIVOT
    v_sql CLOB;

    -- Variable para almacenar el resultado de la consulta PIVOT
    v_result SYS_REFCURSOR;

     -- Variables para almacenar los valores dinámicos de las columnas del PIVOT
    v_aporte        NUMBER;
    v_acumulado     NUMBER;


BEGIN
    --***************************************************************
    --OBTENCIÓN DE DATOS
    --***************************************************************
    -- Ejecutar consulta del ID_PERIODO_ANUAL y  ANO para almacenarlos en arreglos para crear el PIVOT 
    FOR rec IN (SELECT PERIODO_ANUAL FROM TABLE(PPI.OBTENER_PERIODO_ANUAL_DISTRIBUCION_X_PERIODO_ANUAL(periodoAnual)) ORDER BY PERIODO_ANUAL)
    LOOP

        -- Utilizar el constructor del tipo ANO_TYPE
        v_periodo := PERIODO_ANUAL_TYPE(rec.PERIODO_ANUAL);

       -- Añadir los resultados al arreglo de años
        v_periodos.EXTEND;
        v_periodos(v_periodos.LAST) := v_periodo;



    END LOOP;


    --**************************************************************
    --CONSULTA DINÁMICA
    --**************************************************************
    v_sql := 'SELECT ';
    
    --FOR i IN 1..v_periodos.LAST LOOP
     --   v_sql := v_sql || 'NVL(EJECUCION_'|| v_periodos(i).PERIODO_ANUAL || '.EJECUCION,0) EJECUCION_' || v_periodos(i).PERIODO_ANUAL || ',';   
    --END LOOP;
    v_sql := v_sql || 'EJECUCION_' || v_periodos(1).PERIODO_ANUAL || '.APORTE_ESTRATEGICO,';
   
    v_sql := v_sql || '(';
    FOR i IN 1..v_periodos.LAST LOOP
        v_sql := v_sql || 'NVL(EJECUCION_'|| v_periodos(i).PERIODO_ANUAL || '.EJECUCION,0) ';
        IF i < v_periodos.LAST THEN
            v_sql := v_sql || ' + ';
        END IF;
    END LOOP;
    v_sql := v_sql || ') AS ACUMULADO ';
    v_sql := v_sql || 'FROM TABLE(PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL(' || v_periodos(1).PERIODO_ANUAL || ')) EJECUCION_' || v_periodos(1).PERIODO_ANUAL || ' ';
    
    FOR i IN 2..v_periodos.LAST LOOP
        v_sql := v_sql || 'LEFT JOIN ';  
        v_sql := v_sql || '( ';  
        v_sql := v_sql || 'SELECT '; 
        v_sql := v_sql || 'APORTE_ESTRATEGICO, '; 
        v_sql := v_sql || 'EJECUCION '; 
        v_sql := v_sql || 'FROM TABLE(PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL(' || v_periodos(i).PERIODO_ANUAL || '))'; 
        v_sql := v_sql || ') EJECUCION_' ||  v_periodos(i).PERIODO_ANUAL || ' ON EJECUCION_' || v_periodos(1).PERIODO_ANUAL || '.APORTE_ESTRATEGICO = EJECUCION_' || v_periodos(i).PERIODO_ANUAL || '.APORTE_ESTRATEGICO '; 
        v_sql := v_sql || ' '; 
    END LOOP;
    

  

    v_sql := v_sql || 'ORDER BY EJECUCION_'||v_periodos(1).PERIODO_ANUAL ||'.APORTE_ESTRATEGICO';


    -- Ejecutar la consulta PIVOT
    OPEN v_result FOR v_sql;

    --**************************************************************
    --RECUPERAR Y FORMATEAAR LOS DATOS
    --**************************************************************
    LOOP
       FETCH v_result INTO v_aporte,v_acumulado;
       EXIT WHEN v_result%NOTFOUND;

       PIPE ROW(ACUMULADO_TYPE(v_aporte,v_acumulado));
    END LOOP;

    CLOSE v_result;

    RETURN;

END OBTENER_ACUMULADO_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_ACUMULADO_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_X_PERIODO_ANUAL(:periodoAnual));








--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.ACUMULADO_ACCION_TABLE_TYPE;
DROP TYPE PPI.ACUMULADO_ACCION_TYPE;
DROP FUNCTION PPI.OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_ACCION_TYPE AS OBJECT (
    ACCION_ESTRATEGICA          NUMBER,
    ACUMULADO_ACCION            NUMBER
);

GRANT EXECUTE ON PPI.ACUMULADO_ACCION_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_ACCION_TABLE_TYPE AS TABLE OF ACUMULADO_ACCION_TYPE;

GRANT EXECUTE ON PPI.ACUMULADO_ACCION_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN ACUMULADO_ACCION_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT 
              ACE.ID_ACCION_ESTRATEGICA,
              SUM(ACUMULADO.ACUMULADO) AS ACUMULADO_X_ACCION
            FROM 
              ACCION_ESTRATEGICA ACE
            INNER JOIN 
              APORTE_ESTRATEGICO APE ON (APE.ACCION_ESTRATEGICA = ACE.ID_ACCION_ESTRATEGICA)
            -- ACUMULADO
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_X_PERIODO_ANUAL(periodoAnual))
            ) ACUMULADO ON APE.ID_APORTE_ESTRATEGICO = ACUMULADO.APORTE_ESTRATEGICO
            GROUP BY 
              ACE.ID_ACCION_ESTRATEGICA
            ORDER BY 
              ACE.ID_ACCION_ESTRATEGICA
    ) 
    LOOP
    PIPE ROW (ACUMULADO_ACCION_TYPE(rec.ID_ACCION_ESTRATEGICA,rec.ACUMULADO_X_ACCION));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL(:periodoAnual));



--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.ACUMULADO_METAPMP_TABLE_TYPE;
DROP TYPE PPI.ACUMULADO_METAPMP_TYPE;
DROP FUNCTION PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_METAPMP_TYPE AS OBJECT (
    META_PMP                    NUMBER,
    ACUMULADO_METAPMP           NUMBER
);

GRANT EXECUTE ON PPI.ACUMULADO_METAPMP_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_METAPMP_TABLE_TYPE AS TABLE OF ACUMULADO_METAPMP_TYPE;

GRANT EXECUTE ON PPI.ACUMULADO_METAPMP_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN ACUMULADO_METAPMP_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT 
              MPMP.ID_META_PMP,
              NVL(AVG(ACUMULADO_ACCION.ACUMULADO_ACCION),0) AS ACUMULADO_X_META
            FROM 
            META_PMP MPMP
            INNER JOIN ACCION_ESTRATEGICA ACE ON (ACE.META_PMP = MPMP.ID_META_PMP)
            --ACUMULADO ACCION
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL(periodoAnual))
            ) ACUMULADO_ACCION ON ACE.ID_ACCION_ESTRATEGICA = ACUMULADO_ACCION.ACCION_ESTRATEGICA
            
            GROUP BY 
              MPMP.ID_META_PMP
            ORDER BY
            MPMP.ID_META_PMP
    ) 
    LOOP
    PIPE ROW (ACUMULADO_METAPMP_TYPE(rec.ID_META_PMP,rec.ACUMULADO_X_META));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL(:periodoAnual));
















--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.ACUMULADO_PRIORIDAD_TABLE_TYPE;
DROP TYPE PPI.ACUMULADO_PRIORIDAD_TYPE;
DROP FUNCTION PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_PRIORIDAD_TYPE AS OBJECT (
    PRIORIDAD_ESTRATEGICA_PMP                    NUMBER,
    ACUMULADO_PRIORIDAD                          NUMBER
);

GRANT EXECUTE ON PPI.ACUMULADO_PRIORIDAD_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_PRIORIDAD_TABLE_TYPE AS TABLE OF ACUMULADO_PRIORIDAD_TYPE;

GRANT EXECUTE ON PPI.ACUMULADO_PRIORIDAD_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN ACUMULADO_PRIORIDAD_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT 
                PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP,
                NVL(AVG(ACUMULADO_METAPMP.ACUMULADO_METAPMP),0) AS ACUMULADO_PRIORIDAD
            FROM 
            PRIORIDAD_ESTRATEGICA_PMP PEPMP
            INNER JOIN DETALLE_PMP DPMP ON (DPMP.PRIORIDAD_ESTRATEGICA_PMP = PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP)
            INNER JOIN META_PMP MPMP ON (MPMP.DETALLE_PMP = DPMP.ID_DETALLE_PMP)
            
            --ACUMULADO METAPMP
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL(periodoAnual))
            ) ACUMULADO_METAPMP ON MPMP.ID_META_PMP = ACUMULADO_METAPMP.META_PMP
            GROUP BY 
              PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP
            ORDER BY
            PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP
    ) 
    LOOP
    PIPE ROW (ACUMULADO_PRIORIDAD_TYPE(rec.ID_PRIORIDAD_ESTRATEGICA_PMP,rec.ACUMULADO_PRIORIDAD));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL(:periodoAnual));












--*******************************************************************************
--*******************************************************************************
-- FUNCIÓN: OBTENER_AVANCE_PPE
--*******************************************************************************
--*******************************************************************************

--+++++++++++++++++++++++++++++++++++++++++++++++++++++
-- PASO 0: ELIMINACIÓN DE DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
DROP TYPE PPI.AVANCE_PPE_TABLE_TYPE;
DROP TYPE PPI.AVANCE_PPE_TYPE;
DROP FUNCTION PPI.OBTENER_AVANCE_PPE_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato APD_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.AVANCE_PPE_TABLE_TYPE AS OBJECT (
    PRIORIDAD_ESTRATEGICA_PMP                    NUMBER,
    ACUMULADO_PRIORIDAD                          NUMBER
);

GRANT EXECUTE ON PPI.ACUMULADO_PRIORIDAD_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.ACUMULADO_PRIORIDAD_TABLE_TYPE AS TABLE OF ACUMULADO_PRIORIDAD_TYPE;

GRANT EXECUTE ON PPI.ACUMULADO_PRIORIDAD_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN ACUMULADO_PRIORIDAD_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT 
                PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP,
                NVL(AVG(ACUMULADO_METAPMP.ACUMULADO_METAPMP),0) AS ACUMULADO_PRIORIDAD
            FROM 
            PRIORIDAD_ESTRATEGICA_PMP PEPMP
            INNER JOIN DETALLE_PMP DPMP ON (DPMP.PRIORIDAD_ESTRATEGICA_PMP = PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP)
            INNER JOIN META_PMP MPMP ON (MPMP.DETALLE_PMP = DPMP.ID_DETALLE_PMP)
            
            --ACUMULADO METAPMP
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL(periodoAnual))
            ) ACUMULADO_METAPMP ON MPMP.ID_META_PMP = ACUMULADO_METAPMP.META_PMP
            GROUP BY 
              PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP
            ORDER BY
            PEPMP.ID_PRIORIDAD_ESTRATEGICA_PMP
    ) 
    LOOP
    PIPE ROW (ACUMULADO_PRIORIDAD_TYPE(rec.ID_PRIORIDAD_ESTRATEGICA_PMP,rec.ACUMULADO_PRIORIDAD));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL(:periodoAnual));


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 01: Crear TIPO de tipo de dato AVANCE_PPE_TABLE_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.AVANCE_PPE_TYPE AS OBJECT (
    PERIODO_ANUAL                                   NUMBER,
    EJE_PLP_CODIGO                                  NUMBER,
    EJE_PLP_DESCRIPCION                             VARCHAR2(500),
    OBJ_PLP_CODIGO                                  NUMBER,
    OBJ_PLP_DESCRIPCION                             VARCHAR2(500),
    EJE_TRA_CODIGO                                  NUMBER,
    EJE_TRA_DESCRIPCION                             VARCHAR2(500),
    PRI_PMP_CODIGO                                  NUMBER,
    PRI_PMP_DESCRIPCION                             VARCHAR2(500),
    ACUMULADO_PRIORIDAD                             NUMBER,
    OBJ_PMP_CODIGO                                  NUMBER,
    OBJ_PMP_DESCRIPCION                             VARCHAR2(500),
    MPMP_CODIGO                                     NUMBER,
    MPMP_DESCRIPCION                                VARCHAR2(500),
    ACUMULADO_METAPMP                               NUMBER,
    AMBITO_ACCION_UNIVERITARIA                      VARCHAR2(500),
    ENCARGADO_MONITOERO                             VARCHAR2(500),
    EPES_CODIGO                                     NUMBER,
    EPES_NOMBRE                                     VARCHAR2(500),
    TEPES_NOMBRE                                    VARCHAR2(500),
    OPDS_CODIGO                                     NUMBER,
    OPDS_NOMBRE                                     VARCHAR2(500),
    UE_PACE_NOMBRE                                  VARCHAR2(500),
    ACE_CODIGO                                      NUMBER,
    ACE_DESCRIPCION                                 VARCHAR2(500),
    ACUMULADO_ACCION                                NUMBER,
    UE_PAPE_NOMBRE                                  VARCHAR2(500),
    APE_CODIGO                                      NUMBER,
    APE_DESCRIPCION                                 VARCHAR2(500),
    CUANTIFICACION                                  NUMBER,
    LINEA_BASE                                      NUMBER,
    FUENTE_RECURSO                                  VARCHAR2(500),
    JORNADA                                         NUMBER,
    DISTRIBUCION_1                                  NUMBER,
    DISTRIBUCION_2                                  NUMBER,
    DISTRIBUCION_3                                  NUMBER,
    DISTRIBUCION_4                                  NUMBER,
    DISTRIBUCION_5                                  NUMBER,
    CANTIDAD_APORTES                                NUMBER,
    REPRESENTATIVIDAD                               NUMBER,
    PROGRAMACION                                    NUMBER,
    PROMEDIO                                        NUMBER,
    EJECUCION                                       NUMBER,
    ACUMULADO_APORTE                                NUMBER
);


GRANT EXECUTE ON PPI.AVANCE_PPE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 02: Crear TABLA de tipo de dato PROGRAMACION_TYPE
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE TYPE PPI.AVANCE_PPE_TABLE_TYPE AS TABLE OF AVANCE_PPE_TYPE;

GRANT EXECUTE ON PPI.AVANCE_PPE_TABLE_TYPE TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;



--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 03: Crear FUNCIÓN
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
CREATE OR REPLACE FUNCTION  PPI.OBTENER_AVANCE_PPE_X_PERIODO_ANUAL(
    periodoAnual IN NUMBER
)
RETURN ACUMULADO_PRIORIDAD_TABLE_TYPE PIPELINED AS

BEGIN

    FOR rec IN (
            SELECT
                DISTINCT
                PO.PERIODO_ANUAL                            PERIODO_ANUAL,
                EJE_PLP.CODIGO                              EJE_PLP_CODIGO,
                EJE_PLP.DESCRIPCION                         EJE_PLP_DESCRIPCION,
                OBJ_PLP.CODIGO                              OBJ_PLP_CODIGO,
                OBJ_PLP.DESCRIPCION                         OBJ_PLP_DESCRIPCION,
                EJE_TRA.CODIGO                              EJE_TRA_CODIGO,
                EJE_TRA.DESCRIPCION                         EJE_TRA_DESCRIPCION,
                PRI_PMP.CODIGO                              PRI_PMP_CODIGO,
                PRI_PMP.DESCRIPCION                         PRI_PMP_DESCRIPCION,
                ACUMULADO_PRIORIDAD.ACUMULADO_PRIORIDAD     ACUMULADO_PRIORIDAD,
                OBJ_PMP.CODIGO                              OBJ_PMP_CODIGO,
                OBJ_PMP.DESCRIPCION                         OBJ_PMP_DESCRIPCION,
                MPMP.CODIGO                                 MPMP_CODIGO,
                MPMP.DESCRIPCION                            MPMP_DESCRIPCION, 
                ACUMULADO_METAPMP.ACUMULADO_METAPMP         ACUMULADO_METAPMP,
                AAU.NOMBRE                                  AMBITO_ACCION_UNIVERITARIA,
                UE_EM.NOMBRE                                ENCARGADO_MONITOERO,
                EPES.CODIGO                                 EPES_CODIGO,
                EPES.NOMBRE                                 EPES_NOMBRE,
                TEPES.NOMBRE                                TEPES_NOMBRE,
                OPDS.CODIGO                                 OPDS_CODIGO,
                OPDS.NOMBRE                                 OPDS_NOMBRE,
                UE_PACE.NOMBRE                              UE_PACE_NOMBRE,
                ACE.CODIGO                                  ACE_CODIGO,
                ACE.DESCRIPCION                             ACE_DESCRIPCION,
                ACUMULADO_ACCION.ACUMULADO_ACCION           ACUMULADO_ACCION,    
                UE_PAPE.NOMBRE                              UE_PAPE_NOMBRE,
                APE.CODIGO                                  APE_CODIGO,
                APE.DESCRIPCION                             APE_DESCRIPCION,
                APE.CUANTIFICACION                          CUANTIFICACION,
                APE.LINEA_BASE                              LINEA_BASE,
                FR.NOMBRE                                   FUENTE_RECURSO,
                APE.JORNADA                                 JORNADA,
                APD.ANNO_1                                  DISTRIBUCION_1,
                APD.ANNO_2                                  DISTRIBUCION_2,
                APD.ANNO_3                                  DISTRIBUCION_3,
                APD.ANNO_4                                  DISTRIBUCION_4,
                APD.ANNO_5                                  DISTRIBUCION_5,
                PROGRAMACION.CANTIDAD_APORTES               CANTIDAD_APORTES,
                PROGRAMACION.REPRESENTATIVIDAD              REPRESENTATIVIDAD,
                PROGRAMACION.PROGRAMACION                   PROGRAMACION,
                PROMEDIO.PROMEDIO                           PROMEDIO,
                EJECUCION.EJECUCION                         EJECUCION,
                ACUMULADO.ACUMULADO                         ACUMULADO_APORTE
            FROM 
            META_POA_EVALUACION                         MPE
            INNER JOIN META_POA                         MP          ON (MP.ID_META_POA                          =   MPE.META_POA)
            INNER JOIN OBJETIVO_POA                     OP          ON (OP.ID_OBJETIVO_POA                      =   MP.OBJETIVO_POA)
            INNER JOIN PLAN_OPERATIVO                   PO          ON (PO.ID_PLAN_OPERATIVO                    =   OP.PLAN_OPERATIVO)
            INNER JOIN UNIDAD_EJECUTORA                 UE_PO       ON (UE_PO.ID_UNIDAD_EJECUTORA               =   PO.UNIDAD_EJECUTORA)
            INNER JOIN EVALUACION_OPERATIVA             EO          ON (EO.PLAN_OPERATIVO                       =   PO.ID_PLAN_OPERATIVO)
            INNER JOIN META_POA_APORTE                  MPA         ON (MPA.META_POA                            =   MP.ID_META_POA)
            INNER JOIN APORTE_ESTRATEGICO               APE         ON (APE.ID_APORTE_ESTRATEGICO               =   MPA.APORTE_ESTRATEGICO)
            INNER JOIN PLAN_APORTE_ESTRATEGICO          PAPE        ON (PAPE.ID_PLAN_APORTE_ESTRATEGICO         =   APE.PLAN_APORTE_ESTRATEGICO)
            INNER JOIN UNIDAD_EJECUTORA                 UE_PAPE     ON (UE_PAPE.ID_UNIDAD_EJECUTORA             =   PAPE.UNIDAD_EJECUTORA)
            LEFT JOIN  APORTE_PAE_FUENTE_RECURSO        APAEFR      ON (APAEFR.APORTE_ESTRATEGICO               =   APE.ID_APORTE_ESTRATEGICO) --FALTAN
            LEFT JOIN  FUENTE_RECURSO                   FR          ON (FR.ID_FUENTE_RECURSO                    =   APAEFR.FUENTE_RECURSO) --FALTAN
            INNER JOIN ACCION_ESTRATEGICA               ACE         ON (ACE.ID_ACCION_ESTRATEGICA               =   APE.ACCION_ESTRATEGICA)
            INNER JOIN PLAN_ACCION_ESTRATEGICA          PACE        ON (PACE.ID_PLAN_ACCION_ESTRATEGICA         =   ACE.PLAN_ACCION_ESTRATEGICA)
            INNER JOIN UNIDAD_EJECUTORA                 UE_PACE     ON (UE_PACE.ID_UNIDAD_EJECUTORA             =   PACE.UNIDAD_EJECUTORA)
            INNER JOIN META_PMP                         MPMP        ON (MPMP.ID_META_PMP                        =   ACE.META_PMP)
            INNER JOIN AMBITO_ACCION_UNIVERSITARIA      AAU         ON (AAU.ID_AMBITO_ACCION_UNIVERSITARIA      =   MPMP.AMBITO_ACCION_UNIVERSITARIA)
            INNER JOIN ENCARGADO_MONITOREO              EM          ON (EM.ID_ENCARGADO_MONITOREO               =   MPMP.ENCARGADO_MONITOREO)
            INNER JOIN UNIDAD_EJECUTORA                 UE_EM       ON (UE_EM.ID_UNIDAD_EJECUTORA               =   EM.UNIDAD_EJECUTORA)
            LEFT  JOIN META_PMP_EJE_PES                 MPEP        ON (MPEP.META_PMP                           =   MPMP.ID_META_PMP) --VACIA
            LEFT  JOIN EJE_PES                          EPES        ON (EPES.ID_EJE_PES                         =   MPEP.EJE_PES)
            LEFT  JOIN TIPO_EJE_PES                     TEPES       ON (TEPES.ID_TIPO_EJE_PES                   =   EPES.TIPO_EJE_PES)
            LEFT  JOIN META_PMP_OBJETIVO_PDS            MPOPDS      ON (MPOPDS.META_PMP                         =   MPMP.ID_META_PMP) --VACIA
            LEFT  JOIN OBJETIVO_PDS                     OPDS        ON (OPDS.ID_OBJETIVO_PDS                    =   MPOPDS.OBJETIVO_PDS)
            INNER JOIN DETALLE_PMP                      DET_PMP     ON (DET_PMP.ID_DETALLE_PMP                  =   MPMP.DETALLE_PMP)
            INNER JOIN PRIORIDAD_ESTRATEGICA_PMP        PRI_PMP     ON (PRI_PMP.ID_PRIORIDAD_ESTRATEGICA_PMP    =   DET_PMP.PRIORIDAD_ESTRATEGICA_PMP)
            INNER JOIN OBJETIVO_ESTRATEGICO_PMP         OBJ_PMP     ON (OBJ_PMP.ID_OBJETIVO_ESTRATEGICO_PMP     =   DET_PMP.OBJETIVO_ESTRATEGICO_PMP)
            INNER JOIN EJE_ESTRATEGICO_PLP              EJE_PLP     ON (EJE_PLP.ID_EJE_ESTRATEGICO_PLP          =   DET_PMP.EJE_ESTRATEGICO_LARGO_PLAZO)
            INNER JOIN DETALLE_PLP                      DET_PLP     ON (DET_PLP.EJE_ESTRATEGICO_PLP             =   EJE_PLP.ID_EJE_ESTRATEGICO_PLP)   
            INNER JOIN OBJETIVO_ESTRATEGICO_PLP         OBJ_PLP     ON (OBJ_PLP.ID_OBJETIVO_ESTRATEGICO_PLP     =   DET_PLP.OBJETIVO_ESTRATEGICO_PLP)
            INNER JOIN DETALLE_PLP_EJE_TRANSVERSAL      DET_EJE_TRA ON (DET_EJE_TRA.DETALLE_PLP                 =   DET_PLP.ID_DETALLE_PLP)
            LEFT  JOIN EJE_TRANSVERSAL_PLP              EJE_TRA     ON (EJE_TRA.ID_EJE_TRANSVERSAL_PLP          =   DET_EJE_TRA.EJE_TRANSVERSAL_PLP) --VACIA
            INNER JOIN PLAN_LARGO_PLAZO                 PLP         ON (PLP.ID_PLAN_LARGO_PLAZO                 =   DET_PLP.PLAN_LARGO_PLAZO)
            
            --METAS FUNCIONALES
            INNER JOIN (SELECT *
                    FROM TABLE(PPI.OBTENER_METAS_OPE_FUNCIONALES_X_PERIODO_ANUAL(:periodoAnual))
            ) METAS_FUNCIONALES on MPE.META_POA = METAS_FUNCIONALES.META_POA
            
            -- DISTRIBUCION PORCENTUAL
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_APD_X_PERIODO_ANUAL(:periodoAnual))
            ) APD ON APE.ID_APORTE_ESTRATEGICO = APD.APORTE_ESTRATEGICO
            
            -- PROGRAMACION
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_PROGRAMACION_X_PERIODO_ANUAL(:periodoAnual))
            ) PROGRAMACION ON APE.ID_APORTE_ESTRATEGICO = PROGRAMACION.APORTE_ESTRATEGICO
            
            --***************************************
            -- A PARTIR DE SE REQUIERE LA ETAPA_PLAN 
            --***************************************
            
            -- PROMEDIO
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_PROMEDIO_AVANCE_APORTE_X_PERIODO_ANUAL(:periodoAnual))
            ) PROMEDIO ON APE.ID_APORTE_ESTRATEGICO = PROMEDIO.APORTE_ESTRATEGICO AND MPE.ETAPA_PLAN = PROMEDIO.ETAPA_PLAN
            
            --EJECUCION
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_EJECUCION_X_PERIODO_ANUAL(:periodoAnual))
            ) EJECUCION ON APE.ID_APORTE_ESTRATEGICO = EJECUCION.APORTE_ESTRATEGICO AND MPE.ETAPA_PLAN = EJECUCION.ETAPA_PLAN
            
            --ACUMULADO APORTE
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_X_PERIODO_ANUAL(:periodoAnual))
            ) ACUMULADO ON APE.ID_APORTE_ESTRATEGICO = ACUMULADO.APORTE_ESTRATEGICO
            
            --ACUMULADO ACCION
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_ACCION_X_PERIODO_ANUAL(:periodoAnual))
            ) ACUMULADO_ACCION ON ACE.ID_ACCION_ESTRATEGICA = ACUMULADO_ACCION.ACCION_ESTRATEGICA
            
            
            --ACUMULADO METAPMP
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_METAPMP_X_PERIODO_ANUAL(:periodoAnual))
            ) ACUMULADO_METAPMP ON MPMP.ID_META_PMP = ACUMULADO_METAPMP.META_PMP
            
            --ACUMULADO PRIORIDAD
            INNER JOIN (
                SELECT * FROM TABLE(PPI.OBTENER_ACUMULADO_PRIORIDAD_X_PERIODO_ANUAL(:periodoAnual))
            ) ACUMULADO_PRIORIDAD ON ID_PRIORIDAD_ESTRATEGICA_PMP = ACUMULADO_PRIORIDAD.PRIORIDAD_ESTRATEGICA_PMP
            
            WHERE 
            PO.PERIODO_ANUAL = :periodoAnual
            AND MPE.ETAPA_PLAN = EO.ETAPA_PLAN
            ORDER BY 
            EJE_PLP.CODIGO,
            OBJ_PLP.CODIGO,
            EJE_TRA.CODIGO,
            PRI_PMP.CODIGO,
            OBJ_PMP.CODIGO,
            MPMP.CODIGO,
            AAU.NOMBRE,
            UE_EM.NOMBRE,
            EPES.CODIGO,
            OPDS.CODIGO,
            UE_PACE.NOMBRE,
            ACE.CODIGO,
            UE_PAPE.NOMBRE,
            APE.CODIGO
    ) 
    LOOP
    PIPE ROW (AVANCE_PPE_TYPE(
                                rec.ID_PRIORIDAD_ESTRATEGICA_PMP,
                                rec.ACUMULADO_PRIORIDAD
                                
                             ));
   END LOOP;

    DBMS_OUTPUT.PUT_LINE('DEBUG: periodoAnual: ' || periodoAnual);
    
    RETURN;

END OBTENER_AVANCE_PPE_X_PERIODO_ANUAL;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 04: GRANTS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++

--***********************************************
-- GRANT'S
--***********************************************
GRANT EXECUTE ON  PPI.OBTENER_AVANCE_PPE_X_PERIODO_ANUAL TO WWW_SIGESA,ANALISTA, ANALISTA_SIGESA;


--+++++++++++++++++++++++++++++++++++++++++++++++++++++
--PASO 05: CONSULTAR DATOS
--+++++++++++++++++++++++++++++++++++++++++++++++++++++
SELECT * FROM TABLE(PPI.OBTENER_AVANCE_PPE_X_PERIODO_ANUAL(:periodoAnual)) AVANCE_PPE;